import { Component, ElementRef, Input, ViewChild, Output, EventEmitter } from '@angular/core';
import { AbstractControlDirective, NgControl } from '@angular/forms';
import { Observable } from 'rxjs/Observable';
var RlDatePickerInputComponent = (function () {
    function RlDatePickerInputComponent(el, control) {
        var _this = this;
        this.el = el;
        this.control = control;
        this.options = {
            format: 'MM/dd/yyyy',
            minDate: null,
            maxDate: null,
            showWeeks: false
        };
        this.onClose = new EventEmitter();
        this.opened = false;
        this.dateObject = new Date();
        this.ngOnDestroy = function () { return _this.removeEventListener(); };
    }
    RlDatePickerInputComponent.prototype.ngOnInit = function () {
        this.dateObject = this.ngModel || this.control.value;
    };
    RlDatePickerInputComponent.prototype.ngDoCheck = function () {
        var _this = this;
        if (this.dateObject !== this.$dateObject) {
            this.$dateObject = this.dateObject;
            this.control.control.setValue(this.dateObject);
        }
        if (!this.opened && this.subscriber && !this.subscriber.closed) {
            setTimeout(function () {
                _this.removeEventListener();
                _this.onClose.emit();
            }, 0);
        }
        else if (this.opened && (!this.subscriber || (this.subscriber && this.subscriber.closed))) {
            setTimeout(function () {
                _this.addEventListener();
            }, 0);
        }
    };
    Object.defineProperty(RlDatePickerInputComponent.prototype, "LayoutElement", {
        /**
         * Get the HTMLElement of this Element
         * @returns {HTMLElement}
         * @constructor
         */
        get: function () {
            return this.el.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RlDatePickerInputComponent.prototype, "DatepickerElement", {
        /**
         * Get the HTMLElement of the Datepicker
         * @returns {HTMLElement}
         * @constructor
         */
        get: function () {
            return this.LayoutElement.children[1];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RlDatePickerInputComponent.prototype, "OpenBtnElement", {
        get: function () {
            return (this.OpenBtn.nativeElement || this.OpenBtn);
        },
        enumerable: true,
        configurable: true
    });
    RlDatePickerInputComponent.prototype.removeEventListener = function () {
        if (this.subscriber)
            this.subscriber.unsubscribe();
    };
    RlDatePickerInputComponent.prototype.addEventListener = function () {
        var _this = this;
        this.docmousedown = Observable.fromEvent(document, 'mousedown');
        this.subscriber = this.docmousedown.subscribe(function (e) {
            e.stopPropagation();
            e.preventDefault();
            var btnClicked = _this.OpenBtnElement.contains(e.target) === false;
            var dtClicked = _this.DatepickerElement.contains(e.target) === false;
            if (btnClicked && dtClicked)
                _this.opened = false;
        });
    };
    RlDatePickerInputComponent.prototype.open = function () {
        this.opened = !this.opened;
        this.opened ? this.addEventListener() : this.removeEventListener();
    };
    RlDatePickerInputComponent.prototype.setToToday = function () {
        this.dateObject = new Date();
        this.opened = false;
    };
    RlDatePickerInputComponent.prototype.clearDate = function () {
        this.dateObject = undefined;
        this.opened = false;
    };
    __decorate([
        ViewChild('OpenBtn'),
        __metadata("design:type", ElementRef)
    ], RlDatePickerInputComponent.prototype, "OpenBtn", void 0);
    __decorate([
        Input('ngModel'),
        __metadata("design:type", AbstractControlDirective)
    ], RlDatePickerInputComponent.prototype, "ngModel", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], RlDatePickerInputComponent.prototype, "options", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], RlDatePickerInputComponent.prototype, "onClose", void 0);
    RlDatePickerInputComponent = __decorate([
        Component({
            selector: 'rl-datepicker-input',
            template: "\n    <div class=\"input-group\">\n      <input class=\"input-sm form-control\" name=\"datepicker\"\n            [value]=\"dateObject | date: options.format\" type=\"text\" disabled/>\n      <div class=\"input-group-btn\">\n        <button class=\"btn btn-default\" type=\"button\" (click)=\"open()\" #OpenBtn>\n          <span class=\"glyphicon glyphicon-calendar\"></span>\n        </button>\n      </div>\n    </div>\n    <div rlDatepickerPopup>\n      <datepicker [(ngModel)]=\"dateObject\" name=\"datepicker\" [hidden]=\"!opened\" [minDate]=\"options.minDate\"\n                  [maxDate]=\"options.maxDate\" [showWeeks]=\"options.showWeeks\"\n                  (selectionDone)=\"opened = false\"></datepicker>\n      <div [hidden]=\"!opened\" style=\"padding: 5px;\">\n        <span class=\"btn-group pull-left\">       \n          <button type=\"button\" class=\"btn btn-sm btn-info\"\n                  (click)=\"setToToday()\">Today</button>\n          <button type=\"button\" class=\"btn btn-sm btn-danger\"\n                  (click)=\"clearDate()\">Clear</button>\n        </span>\n        <button type=\"button\" class=\"btn btn-sm btn-success pull-right\" (click)=\"opened = false\">Done</button>\n      </div>\n    </div>\n  "
        }),
        __metadata("design:paramtypes", [ElementRef,
            NgControl])
    ], RlDatePickerInputComponent);
    return RlDatePickerInputComponent;
}());
export { RlDatePickerInputComponent };
//# sourceMappingURL=datepicker-input.component.js.map